package jalview.datamodel.xdb.embl;

import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertFalse;
import static org.testng.AssertJUnit.assertTrue;
import static org.testng.AssertJUnit.assertNull;
import org.testng.annotations.Test;
import java.io.StringReader;
import java.util.Vector;

import jalview.datamodel.DBRefEntry;

public class EmblFileTest
{
  // adapted from http://www.ebi.ac.uk/Tools/dbfetch/dbfetch/embl/x53828/emblxml
  private static final String TESTDATA = 
          "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>"
          + "<EMBL_Services>"
          + "<entry accession=\"X53828\" version=\"3\" lastUpdated=\"2005-04-18\" releaseCreated=\"25\" releaseLastUpdated=\"83\">"
          + "<description>Chicken LDH-A mRNA for lactate dehydrogenase A chain (EC 1.1.1.27)</description>"
          + "<keyword>L-lactate dehydrogenase</keyword><keyword>chutney</keyword>"
          + "<dbreference db=\"EuropePMC\" primary=\"PMC1460223\" secondary=\"9649548\" />"
          + "<feature name=\"CDS\"><dbreference db=\"GOA\" primary=\"P00340\" secondary=\"2.1\" /><dbreference db=\"InterPro\" primary=\"IPR001236\" />"
          + "<qualifier name=\"note\"><value>L-lactate dehydrogenase A-chain</value><value>pickle</value></qualifier>"
          + "<qualifier name=\"translation\"><value>MSLKDHLIHN</value><evidence>Keith</evidence></qualifier>"
          + "<location type=\"single\" complement=\"true\">"
              + "<locationElement type=\"range\" accession=\"X53828\" version=\"1\" complement=\"false\">"
          + "<basePosition type=\"simple\">60</basePosition><basePosition type=\"join\">1058</basePosition>"
              + "</locationElement></location></feature>"
          + "<sequence type=\"mRNA\" version=\"2\">GTGACG</sequence></entry></EMBL_Services>";

  @Test
  public void testGetEmblFile()
  {
    Vector<EmblEntry> entries = EmblFile.getEmblFile(
            new StringReader(TESTDATA)).getEntries();
    assertEquals(1, entries.size());
    EmblEntry entry = entries.get(0);

    assertEquals("X53828", entry.getAccession());
    assertEquals(
            "Chicken LDH-A mRNA for lactate dehydrogenase A chain (EC 1.1.1.27)",
            entry.getDesc());
    assertEquals("2005-04-18", entry.getLastUpdated());

    /*
     * FIXME these assertions fail - values are null - why?? Adding or removing
     * attributes in the test XML modifies behaviour. eg. inserting an attribute
     * _before_ lastUpdated results in a null value in this field.
     */
    // assertEquals("25", entry.getRCreated());
    // assertEquals("83", entry.getRLastUpdated());

    assertEquals(2, entry.getKeywords().size());
    assertEquals("L-lactate dehydrogenase", entry.getKeywords().get(0));
    assertEquals("chutney", entry.getKeywords().get(1));

    /*
     * dbrefs
     */
    assertEquals(1, entry.getDbRefs().size());
    DBRefEntry dbref = entry.getDbRefs().get(0);
    assertEquals("EuropePMC", dbref.getSource());
    assertEquals("PMC1460223", dbref.getAccessionId());
    assertEquals("9649548", dbref.getVersion());

    /*
     * sequence features
     */
    assertEquals(1, entry.getFeatures().size());
    EmblFeature ef = entry.getFeatures().get(0);
    assertEquals("CDS", ef.getName());
    assertEquals(2, ef.getDbRefs().size());
    dbref = ef.getDbRefs().get(0);
    assertEquals("GOA", dbref.getSource());
    assertEquals("P00340", dbref.getAccessionId());
    assertEquals("2.1", dbref.getVersion());
    dbref = ef.getDbRefs().get(1);
    assertEquals("InterPro", dbref.getSource());
    assertEquals("IPR001236", dbref.getAccessionId());
    assertEquals("", dbref.getVersion());
    assertEquals(2, ef.getQualifiers().size());

    // feature qualifiers
    Qualifier q = ef.getQualifiers().get(0);
    assertEquals("note", q.getName());
    assertEquals(2, q.getValues().length);
    assertEquals("L-lactate dehydrogenase A-chain", q.getValues()[0]);
    assertEquals("pickle", q.getValues()[1]);
    assertNull(q.getEvidence());
    q = ef.getQualifiers().get(1);
    assertEquals("translation", q.getName());
    assertEquals(1, q.getValues().length);
    assertEquals("MSLKDHLIHN", q.getValues()[0]);
    assertEquals(1, q.getEvidence().length);
    assertEquals("Keith", q.getEvidence()[0]);

    // feature locations
    assertEquals(1, ef.getLocations().size());
    EmblFeatureLocations fl = ef.getLocations().get(0);
    assertEquals("single", fl.getLocationType());
    assertTrue(fl.isLocationComplement());
    assertEquals(1, fl.getLocElements().size());
    EmblFeatureLocElement le = fl.getLocElements().get(0);
    assertEquals("range", le.getType());
    assertEquals("X53828", le.getAccession());
    assertEquals("1", le.getVersion());
    assertFalse(le.isComplement());
    assertEquals(2, le.getBasePositions().length);
    BasePosition bp = le.getBasePositions()[0];
    assertEquals("simple", bp.getType());
    assertEquals("60", bp.getPos());
    bp = le.getBasePositions()[1];
    assertEquals("join", bp.getType());
    assertEquals("1058", bp.getPos());

    /*
     * Sequence
     */
    EmblSequence seq = entry.getSequence();
    assertEquals("mRNA", seq.getType());
    assertEquals("2", seq.getVersion());
    assertEquals("GTGACG", seq.getSequence());
  }
}
